from django.urls import reverse

from django.contrib.auth.mixins import UserPassesTestMixin

from ..models import Page, Program, StaticPage
from .mixins import ParentMixin
from .page import PageDetailView, PageListView, PageUpdateView

__all__ = ["ProgramPageDetailView", "ProgramDetailView", "ProgramPageListView"]


class BaseProgramMixin:
    def get_program(self):
        return self.object

    def get_sidebar_url(self):
        return reverse("program-page-list", kwargs={"parent_slug": self.program.slug})

    def get_context_data(self, **kwargs):
        self.program = self.get_program()
        kwargs["program"] = self.program
        return super().get_context_data(**kwargs)


class ProgramDetailView(BaseProgramMixin, PageDetailView):
    model = Program

    def get_template_names(self):
        return super().get_template_names() + ["aircox/program_detail.html"]

    def get_sidebar_queryset(self):
        return super().get_sidebar_queryset().filter(parent=self.program)


class ProgramUpdateView(UserPassesTestMixin, BaseProgramMixin, PageUpdateView):
    model = Program
    fields = ["content"]

    def get_sidebar_queryset(self):
        return super().get_sidebar_queryset().filter(parent=self.program)

    def test_func(self):
        program = self.get_object()
        return self.request.user.has_perm("aircox.%s" % program.change_permission_codename)


class ProgramListView(PageListView):
    model = Program
    attach_to_value = StaticPage.ATTACH_TO_PROGRAMS


# FIXME: not used
class ProgramPageDetailView(BaseProgramMixin, ParentMixin, PageDetailView):
    """Base view class for a page that is displayed as a program's child
    page."""

    parent_model = Program

    def get_program(self):
        self.parent = self.object.program
        return self.object.program

    def get_sidebar_queryset(self):
        return super().get_sidebar_queryset().filter(parent=self.program)


class ProgramPageListView(BaseProgramMixin, PageListView):
    model = Page
    parent_model = Program
    queryset = Page.objects.select_subclasses()

    def get_program(self):
        return self.parent

    def get_context_data(self, **kwargs):
        kwargs.setdefault("sidebar_url_parent", None)
        return super().get_context_data(**kwargs)
